package com.qmth.themis.business.dao;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.themis.business.bean.backend.*;
import com.qmth.themis.business.dto.response.TEExamUnFinishDto;
import com.qmth.themis.business.entity.TOeExamRecord;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * @Description: 考试记录 Mapper 接口
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Mapper
public interface TOeExamRecordMapper extends BaseMapper<TOeExamRecord> {

    /**
     * 获取考试未完列表
     *
     * @param studentId
     * @param examId
     * @param orgId
     * @return
     */
    public TEExamUnFinishDto getUnFinishExam(@Param("studentId") Long studentId, @Param("examId") Long examId, @Param("orgId") Long orgId);

    /**
     * 数据更新
     *
     * @param recordId
     * @param colName
     * @param colValue
     */
    public void dataUpdate(@Param("recordId") Long recordId, @Param("colName") String colName, @Param("colValue") Object colValue);

    /**
     * 查询实时监控台列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param paperDownload
     * @param status
     * @param name
     * @param identity
     * @param minWarningCount
     * @param maxWarningCount
     * @param clientWebsocketStatus
     * @param monitorStatusSource
     * @param userId
     * @return
     */
    public IPage<InvigilateListBean> invigilatePageList(IPage<Map> iPage, @Param("examId") Long examId,
                                                        @Param("examActivityId") Long examActivityId,
                                                        @Param("roomCode") String roomCode,
                                                        @Param("paperDownload") Integer paperDownload,
                                                        @Param("status") String status,
                                                        @Param("name") String name,
                                                        @Param("identity") String identity,
                                                        @Param("minWarningCount") Integer minWarningCount,
                                                        @Param("maxWarningCount") Integer maxWarningCount,
                                                        @Param("clientWebsocketStatus") String clientWebsocketStatus,
                                                        @Param("monitorStatusSource") String monitorStatusSource,
                                                        @Param("userId") Long userId);

    /**
     * 查询实时监控台视频列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param paperDownload
     * @param status
     * @param name
     * @param identity
     * @param minWarningCount
     * @param maxWarningCount
     * @param clientWebsocketStatus
     * @param userId
     * @param orgId
     * @return
     */
    public IPage<InvigilateListVideoBean> invigilatePageListVideo(IPage<Map> iPage, @Param("examId") Long examId,
                                                                  @Param("examActivityId") Long examActivityId,
                                                                  @Param("roomCode") String roomCode,
                                                                  @Param("paperDownload") Integer paperDownload,
                                                                  @Param("status") String status,
                                                                  @Param("name") String name,
                                                                  @Param("identity") String identity,
                                                                  @Param("minWarningCount") Integer minWarningCount,
                                                                  @Param("maxWarningCount") Integer maxWarningCount,
                                                                  @Param("clientWebsocketStatus") String clientWebsocketStatus,
                                                                  @Param("userId") Long userId,
                                                                  @Param("orgId") Long orgId);

    /**
     * 实时监控台视频随机列表
     *
     * @param examId
     * @param userId
     * @param randomNum
     * @param orgId
     * @return
     */
    public List<InvigilateListVideoBean> invigilatePageListVideoRandom(@Param("examId") Long examId, @Param("userId") Long userId, @Param("randomNum") Integer randomNum, @Param("orgId") Long orgId);

    /**
     * 查询在线巡考列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param status
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param clientWebsocketStatus
     * @param userId
     * @param orgId
     * @return
     */
    public IPage<InvigilateListPatrolBean> invigilatePagePatrolList(IPage<Map> iPage, @Param("examId") Long examId,
                                                                    @Param("examActivityId") Long examActivityId,
                                                                    @Param("roomCode") String roomCode,
                                                                    @Param("status") String status,
                                                                    @Param("name") String name,
                                                                    @Param("identity") String identity,
                                                                    @Param("minMultipleFaceCount") Integer minMultipleFaceCount,
                                                                    @Param("maxMultipleFaceCount") Integer maxMultipleFaceCount,
                                                                    @Param("minExceptionCount") Integer minExceptionCount,
                                                                    @Param("maxExceptionCount") Integer maxExceptionCount,
                                                                    @Param("minWarningCount") Integer minWarningCount,
                                                                    @Param("maxWarningCount") Integer maxWarningCount,
                                                                    @Param("clientWebsocketStatus") String clientWebsocketStatus,
                                                                    @Param("userId") Long userId,
                                                                    @Param("orgId") Long orgId);

    /**
     * 预警提醒列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param approveStatus
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param userId
     * @param orgId
     * @return
     */
    public IPage<InvigilateListWarningBean> invigilatePageWarningList(IPage<Map> iPage, @Param("examId") Long examId,
                                                                      @Param("examActivityId") Long examActivityId,
                                                                      @Param("roomCode") String roomCode,
                                                                      @Param("approveStatus") Integer approveStatus,
                                                                      @Param("name") String name,
                                                                      @Param("identity") String identity,
                                                                      @Param("minMultipleFaceCount") Integer minMultipleFaceCount,
                                                                      @Param("maxMultipleFaceCount") Integer maxMultipleFaceCount,
                                                                      @Param("minExceptionCount") Integer minExceptionCount,
                                                                      @Param("maxExceptionCount") Integer maxExceptionCount,
                                                                      @Param("minWarningCount") Integer minWarningCount,
                                                                      @Param("maxWarningCount") Integer maxWarningCount,
                                                                      @Param("userId") Long userId,
                                                                      @Param("orgId") Long orgId);

    /**
     * 预警提醒清除未阅列表
     *
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param approveStatus
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param userId
     * @param orgId
     * @return
     */
    public Integer approveStatusListUpdate(@Param("examId") Long examId,
                                           @Param("examActivityId") Long examActivityId,
                                           @Param("roomCode") String roomCode,
                                           @Param("approveStatus") Integer approveStatus,
                                           @Param("name") String name,
                                           @Param("identity") String identity,
                                           @Param("minMultipleFaceCount") Integer minMultipleFaceCount,
                                           @Param("maxMultipleFaceCount") Integer maxMultipleFaceCount,
                                           @Param("minExceptionCount") Integer minExceptionCount,
                                           @Param("maxExceptionCount") Integer maxExceptionCount,
                                           @Param("minWarningCount") Integer minWarningCount,
                                           @Param("maxWarningCount") Integer maxWarningCount,
                                           @Param("userId") Long userId,
                                           @Param("orgId") Long orgId);

    /**
     * 进度查询列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param courseCode
     * @param name
     * @param identity
     * @param userId
     * @param orgId
     * @return
     */
    public IPage<InvigilateListProgressBean> invigilatePageProgressList(IPage<Map> iPage, @Param("examId") Long examId,
                                                                        @Param("examActivityId") Long examActivityId,
                                                                        @Param("roomCode") String roomCode,
                                                                        @Param("courseCode") String courseCode,
                                                                        @Param("name") String name,
                                                                        @Param("identity") String identity,
                                                                        @Param("userId") Long userId,
                                                                        @Param("orgId") Long orgId);

    /**
     * 进度查询列表导出
     *
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param courseCode
     * @param name
     * @param identity
     * @param userId
     * @param orgId
     * @return
     */
    public List<InvigilateListProgressExcelBean> invigilatePageProgressListExport(@Param("examId") Long examId,
                                                                                  @Param("examActivityId") Long examActivityId,
                                                                                  @Param("roomCode") String roomCode,
                                                                                  @Param("courseCode") String courseCode,
                                                                                  @Param("name") String name,
                                                                                  @Param("identity") String identity,
                                                                                  @Param("userId") Long userId,
                                                                                  @Param("orgId") Long orgId);

    /**
     * 监考明细管理列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param courseCode
     * @param status
     * @param breachStatus
     * @param finishType
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param userId
     * @param orgId
     * @return
     */
    public IPage<InvigilateListHistoryBean> invigilatePageListHistory(IPage<Map> iPage, @Param("examId") Long examId,
                                                                      @Param("examActivityId") Long examActivityId,
                                                                      @Param("roomCode") String roomCode,
                                                                      @Param("courseCode") String courseCode,
                                                                      @Param("status") String status,
                                                                      @Param("breachStatus") Integer breachStatus,
                                                                      @Param("finishType") String finishType,
                                                                      @Param("name") String name,
                                                                      @Param("identity") String identity,
                                                                      @Param("minMultipleFaceCount") Integer minMultipleFaceCount,
                                                                      @Param("maxMultipleFaceCount") Integer maxMultipleFaceCount,
                                                                      @Param("minExceptionCount") Integer minExceptionCount,
                                                                      @Param("maxExceptionCount") Integer maxExceptionCount,
                                                                      @Param("minWarningCount") Integer minWarningCount,
                                                                      @Param("maxWarningCount") Integer maxWarningCount,
                                                                      @Param("userId") Long userId,
                                                                      @Param("orgId") Long orgId);

    public List<Map<String, Object>> getDoneCount(@Param("orgId")Long orgId,@Param("examId") Long examId, @Param("activityId") Long activityId,
                                                  @Param("roomCode") String roomCode, @Param("courseCode") String courseCode);

    public List<Map<String, Object>> getDoneCountByDay(@Param("orgId")Long orgId,@Param("examId") Long examId, @Param("activityId") Long activityId,
                                                       @Param("roomCode") String roomCode, @Param("courseCode") String courseCode);

    public Long getCountByExamId(@Param("examId") Long examId);

    public List<TOeExamRecord> getListByExamIdAndStartId(@Param("examId") Long examId, @Param("startId") Long startId);

    public void updateObjectiveScore(@Param("recordId") Long recordId, @Param("score") Double score);

    public TOeExamRecord findOneByPaperId(@Param("paperId") Long paperId);

    /**
     * 在线巡考报表接口
     *
     * @param examId
     * @param userId
     * @return
     */
    public List<InvigilateListPatrolReportBean> patrolReport(@Param("orgId") Long orgId,@Param("examId") Long examId, @Param("userId") Long userId);

    public TOeExamRecord findOneByExamId(@Param("examId") Long examId, @Param("status") String status);

    public Long getExamingCount(@Param("orgId") Long orgId);

    public Long getOnlineCount(@Param("orgId") Long orgId);

    public Long getExceptionCount(@Param("orgId") Long orgId);

    public Long getWarnCount(@Param("orgId") Long orgId);

    public List<Map<String, Object>> getOrgExamingCount(@Param("orgId") Long orgId);

    public List<Map<String, Object>> getOrgDistribution(@Param("orgId") Long orgId);

    public List<Map<String, Object>> getTypeDistribution(@Param("orgId") Long orgId);

    public List<ExaminationMonitorHourWarnCountBean> getWarnTrend(@Param("orgId") Long orgId,@Param("startTime") Long startTime);

    public void updateHasAnswerFile(@Param("recordId") Long recordId, @Param("hasAnswerFile") Integer hasAnswerFile);

    public List<OpenRecordNeedMarkBean> findExamRecordNeedMark(@Param("examId") Long examId,@Param("examStudentIds") List<Long> examStudentIds);
}
