package com.qmth.themis.business.templete.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import com.qmth.themis.business.annotation.ExcelDynamicExport;
import com.qmth.themis.business.annotation.ExcelNotExport;
import com.qmth.themis.business.annotation.ExcelNote;
import com.qmth.themis.business.cache.bean.ObjectiveAnswerCacheBean;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.ExcelDto;
import com.qmth.themis.business.dto.MarkResultSimpleExportDto;
import com.qmth.themis.business.dto.MarkResultStandardExportDto;
import com.qmth.themis.business.entity.TOeExamAnswer;
import com.qmth.themis.business.enums.AnswerTypeEnum;
import com.qmth.themis.business.enums.QuestionTypeEnum;
import com.qmth.themis.business.service.TEExamPaperService;
import com.qmth.themis.business.service.TEExamStudentService;
import com.qmth.themis.business.service.TOeExamAnswerService;
import com.qmth.themis.business.templete.TaskExportCommon;
import com.qmth.themis.business.templete.TaskExportTemplete;
import com.qmth.themis.business.templete.service.TempleteLogicService;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @Description: 成绩标准版导出任务
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/10/20
 */
@Service
public class TaskMarkResultStandardExportTemplete implements TaskExportTemplete {
    private final static Logger log = LoggerFactory.getLogger(TaskMarkResultStandardExportTemplete.class);


    @Resource
    TEExamStudentService teExamStudentService;

    @Resource
    TempleteLogicService templeteLogicService;

    @Resource
    TOeExamAnswerService tOeExamAnswerService;

    @Resource
    TEExamPaperService teExamPaperService;

    /**
     * 考场导出模版
     *
     * @param map
     * @return
     */
    @Override
    public Result exportTask(Map<String, Object> map) throws IOException {
        log.info("开始导出考场数据");
        long start = System.currentTimeMillis();
        TaskExportCommon taskExportCommon = new TaskExportCommon(map);
        taskExportCommon.init();
        String timeFormat = taskExportCommon.getTimeFormat();
        taskExportCommon.setTxtList(new ArrayList());
        FileOutputStream fos = null;
        SXSSFWorkbook wb = null;
        File mkdir = null, file = null;
        int y = 0, min = 0;
        try {
            taskExportCommon.getTxtList().add(DateUtil.format(new Date(), timeFormat) + "->开始准备处理导出的成绩标准版数据");
            Long examId = (Long) map.get("examId");
            Long activityId = (Long) map.get("activityId");
            String identity = (String) map.get("identity");
            String name = (String) map.get("name");
            String courseCode = (String) map.get("courseCode");
            List<MarkResultSimpleExportDto> markResultSimpleExportDtoList = teExamStudentService.markResultQueryExport(examId, activityId, identity, name, courseCode);
            Gson gson = new Gson();
            List<MarkResultStandardExportDto> markResultStandardExportDtoList = gson.fromJson(gson.toJson(markResultSimpleExportDtoList), new TypeToken<List<MarkResultStandardExportDto>>() {
            }.getType());
            Map<Long, List<ExcelDto>> excelDtoMap = null;
            Map<Long, Map<String, ObjectiveAnswerCacheBean>> paperObjectiveAnswerMap = new HashMap<>();
            List<ExcelDto> dynamicExcelHead = new ArrayList<>();
            if (Objects.nonNull(markResultStandardExportDtoList) && markResultStandardExportDtoList.size() > 0) {
                for (MarkResultStandardExportDto m : markResultStandardExportDtoList) {
                    excelDtoMap = new HashMap<>();
                    Map<String, ObjectiveAnswerCacheBean> objectiveAnswerCacheBeanMap = null;
                    if (Objects.nonNull(m.getPaperId())) {//处理试卷
                        if (!paperObjectiveAnswerMap.containsKey(m.getPaperId())) {
                            objectiveAnswerCacheBeanMap = teExamPaperService.getObjectiveAnswerCacheBean(m.getPaperId());
                            paperObjectiveAnswerMap.put(m.getPaperId(), objectiveAnswerCacheBeanMap);
                        } else {
                            objectiveAnswerCacheBeanMap = paperObjectiveAnswerMap.get(m.getPaperId());
                        }
                        if (dynamicExcelHead.size() == 0) {
                            objectiveAnswerCacheBeanMap.forEach((k, v) -> {
                                ObjectiveAnswerCacheBean objectiveAnswerCacheBean = v;
                                String title = QuestionTypeEnum.convertToTitle(objectiveAnswerCacheBean.getStructType()) + k;
                                ExcelDto excelAnswerDto = new ExcelDto(title + "作答");
                                ExcelDto excelScoreDto = new ExcelDto(title + "得分");
                                dynamicExcelHead.add(excelAnswerDto);
                                dynamicExcelHead.add(excelScoreDto);
                            });
                        }
                    } else {
                        continue;
                    }
                    if (Objects.nonNull(m.getRecordId())) {//处理考生答案
                        QueryWrapper<TOeExamAnswer> tOeExamAnswerQueryWrapper = new QueryWrapper<>();
                        tOeExamAnswerQueryWrapper.lambda().eq(TOeExamAnswer::getExamRecordId, m.getRecordId());
                        List<TOeExamAnswer> tOeExamAnswerList = tOeExamAnswerService.list(tOeExamAnswerQueryWrapper);
                        for (TOeExamAnswer s : tOeExamAnswerList) {
                            String questionsTitle = s.getMainNumber() + "_" + s.getSubNumber();
                            if (Objects.nonNull(s.getAnswer())) {
                                ObjectiveAnswerCacheBean objectiveAnswerCacheBean = objectiveAnswerCacheBeanMap.get(questionsTitle);
                                String title = QuestionTypeEnum.convertToTitle(objectiveAnswerCacheBean.getStructType()) + questionsTitle;
                                String answer = null;
                                if (objectiveAnswerCacheBean.getStructType().intValue() == 1 || objectiveAnswerCacheBean.getStructType().intValue() == 2) {
                                    List<String> list = Arrays.asList(s.getAnswer().replace("[", "").replace("]", "").replaceAll(" ", "").replaceAll("\n", "").split(","));
                                    List<String> answerList = new ArrayList<>();
                                    for (String s1 : list) {
                                        answerList.add(AnswerTypeEnum.convertToTitle(s1.toUpperCase()));
                                    }
                                    answer = answerList.toString();
                                } else if (objectiveAnswerCacheBean.getStructType().intValue() == 3) {
                                    answer = AnswerTypeEnum.convertToTitle(s.getAnswer().toUpperCase());
                                }
                                ExcelDto excelAnswerDto = new ExcelDto(title + "作答", answer);
                                ExcelDto excelScoreDto = new ExcelDto(title + "得分", String.valueOf(s.getScore()));
                                List<ExcelDto> excelDtoList = null;
                                if (!excelDtoMap.containsKey(m.getExamStudentId())) {
                                    excelDtoList = new ArrayList<>();
                                } else {
                                    excelDtoList = excelDtoMap.get(m.getExamStudentId());
                                }
                                excelDtoList.add(excelAnswerDto);
                                excelDtoList.add(excelScoreDto);
                                excelDtoMap.put(m.getExamStudentId(), excelDtoList);
                            }
                        }
                    }
                    m.setAnswerExcetDto(excelDtoMap);
                }
            }
            boolean oss = (boolean) taskExportCommon.getOssEnv().get(SystemConstant.OSS);
            StringJoiner stringJoiner = new StringJoiner("");
            if (!oss) {
                stringJoiner.add(SystemConstant.FILES_DIR).add(File.separator);
            }
            LocalDateTime nowTime = LocalDateTime.now();
            String uploadType = String.valueOf(taskExportCommon.getOssEnv().get(SystemConstant.UPLOAD_TYPE));
            stringJoiner.add(uploadType).add(File.separator).add(String.valueOf(nowTime.getYear()))
                    .add(File.separator).add(String.valueOf(nowTime.getMonthValue()))
                    .add(File.separator).add(String.valueOf(nowTime.getDayOfMonth()));
            mkdir = new File(stringJoiner.toString());
            if (!mkdir.exists()) {
                mkdir.mkdirs();
            }
            //创建excel
            wb = new SXSSFWorkbook();
            Sheet sheet = wb.createSheet("成绩统计标准版");
            CellStyle style = wb.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER); // 水平居中格式
            style.setVerticalAlignment(VerticalAlignment.CENTER); //垂直居中
            Row row = sheet.createRow(0);
            Field[] fields = MarkResultStandardExportDto.class.getDeclaredFields();
            int num = 0;
            //绘制表头
            for (int i = 0; i < fields.length; i++) {
                Field field = fields[i];
                field.setAccessible(true);
                Annotation annotation = field.getAnnotation(ExcelNote.class);
                if (Objects.nonNull(annotation)) {
                    Cell cell = row.createCell(i);
                    cell.setCellValue(field.getAnnotation(ExcelNote.class).value());
                    cell.setCellStyle(style);
                    num = i;
                }
            }
            num++;
            Map<String, Integer> position = new HashMap<>();
            //动态表头
            for (ExcelDto excelDto : dynamicExcelHead) {
                Cell cell = row.createCell(num);
                cell.setCellValue(excelDto.getTitle());
                cell.setCellStyle(style);
                sheet.setColumnWidth(num, 15 * 256);
                position.put(excelDto.getTitle(), num);
                num++;
            }
            int cellIndex = 0, max = SystemConstant.MAX_EXPORT_SIZE, size = markResultStandardExportDtoList.size();
            if (max >= size) {
                max = size;
            }
            while (max <= size) {
                List subList = markResultStandardExportDtoList.subList(min, max);
                //绘制数据
                for (; y < subList.size(); y++) {
                    MarkResultStandardExportDto markResultStandardExportDto = (MarkResultStandardExportDto) subList.get(y);
                    int firstRow = y + 1;
                    cellIndex = 0;
                    Row sxssfRow = sheet.createRow(firstRow);
                    for (Field field : fields) {
                        field.setAccessible(true);
                        Annotation annotation = field.getAnnotation(ExcelNotExport.class);
                        if (Objects.isNull(annotation)) {
                            Annotation dynamicExport = field.getAnnotation(ExcelDynamicExport.class);
                            if (Objects.nonNull(dynamicExport)) {
                                Map<Long, List<ExcelDto>> answerExcetDto = markResultStandardExportDto.getAnswerExcetDto();
                                if (Objects.nonNull(answerExcetDto)) {
                                    List<ExcelDto> excelDtoList = answerExcetDto.get(markResultStandardExportDto.getExamStudentId());
                                    excelDtoList.forEach(s -> {
                                        taskExportCommon.createExcelCell(sxssfRow, position.get(s.getTitle()), s.getContent(), style);
                                    });
                                }
                            } else {
                                taskExportCommon.createExcelCell(sxssfRow, cellIndex, field.get(markResultStandardExportDto), style);
                                cellIndex++;
                            }
                        }
                    }
                }
                map.put("max", max);
                map.put("min", min);
                map.put("size", size);
                map = templeteLogicService.execExportMarkResultSimpleLogic(map);
                if (max == size) {
                    break;
                }
                min = max;
                max += SystemConstant.MAX_EXPORT_SIZE;
                if (max >= size) {
                    max = size;
                }
            }
            file = taskExportCommon.createExcel(mkdir.getPath());
            //写入excel并上传附件
            fos = new FileOutputStream(file);
            wb.write(fos);
            taskExportCommon.ossUpload(file);
            long end = System.currentTimeMillis();
            log.info("导出成绩标准版数据结束,============耗时============:{}秒", (end - start) / 1000);
        } catch (Exception e) {
            log.error("导出报错", e);
            taskExportCommon.exception(min, y, e, taskExportCommon.getTxtList());
        } finally {
            if (Objects.nonNull(wb)) {
                wb.dispose();
            }
            if (Objects.nonNull(fos)) {
                fos.flush();
                fos.close();
            }
            //这里写入txt文件
            taskExportCommon.writeExportResultTxt(mkdir.getPath(), taskExportCommon.getTxtList().toString(), file);
        }
        return ResultUtil.ok(Collections.singletonMap(SystemConstant.SUCCESS, true));
    }
}
